import Image
import xml.sax.saxutils
from math import sin, cos, pi, sqrt, atan

def saveLevel(s, name):
    safename = ''
    for c in name:
        if c.isalnum() or (c == ' '): safename += c
    img = Image.new('RGBA', (33, 25), (0x00, 0x00, 0x00, 0xFF))
    #rotated = Image.new('RGBA', (33, 25), (0x00, 0x00, 0x00, 0x00))
    blocks, objs = s.split('|')
    x = 1
    y = 1
    for c in blocks:
        if c == '1':
            img.putpixel((x, y), (0x00, 0x00, 0x00, 0xFF))
        elif c in '2345':
            oldX = float(x-16)
            oldY = float(y-12)*-1
            angle = atan(y/x)
            distance = sqrt(x*x + y*y)
            newAngle = angle - pi/4.0
            newX = int(cos(newAngle)*distance)+16
            newY = int(sin(newAngle)*distance*-1)+12
            #if (newX >= 0) and (newY >= 0) and (newX < 33) and (newY < 25):
                #rotated.putpixel((newX, newY), (0x00, 0x00, 0x00, 0xFF))
        else:
            img.putpixel((x, y), (0x00, 0x00, 0x00, 0x00))
        y += 1
        if y > 23:
            y = 1
            x += 1
    for obj in objs.split('!'):
        if '^' not in obj: continue
        objid, params = obj.split('^')
        params = map(float, params.split(','))
        pos = (int(params[0]/24), int(params[1]/24))
        safe = img.getpixel(pos) in ((0, 0, 0, 0), (0, 0, 0, 1))
        if objid == '1' and safe:
            # Bounceblock/shrapnel
            img.putpixel(pos, (0x11, 0x11, 0x11, 0xFF))
        elif objid == '4' and safe:
            # Floorguard/sticky
            img.putpixel(pos, (0xFF, 0x00, 0x80, 0xFF))
        elif objid == '5':
            # Ninja/Jumpman
            img.putpixel(pos, (0xFF, 0x7F, 0x00, 0xFF))
        elif objid == '6' and safe:
            # Drone/swoop
            img.putpixel(pos, (0xFF, 0x00, 0xB0, 0xFF))
        elif objid == '9' and safe:
            if params[6]:
                # Door and lock/disappearing shrapnel and bomb
                img.putpixel(pos, (0x00, 0xFF, 0xFF, 0xFF))
                img.putpixel((int(params[4]), int(params[5])), (0x11, 0x11, 0x12, 0xFF))
            else:
                img.putpixel((int(params[4]), int(params[5])), (0xFF, 0xFF, 0x00, 0xFF))
        elif objid == '11':
            # Exit and switch/exit and ?
            img.putpixel(pos, (0x00, 0x00, 0xFF, 0xFF))
        elif objid == '12' and safe:
            # Mine/lava
            img.putpixel(pos, (0xDD, 0xDD, 0xDD, 0xFF))
    img.save('n.jmp/%s.png' % safename)
    #rotated.save('n.jmp/%s2.png' % safename)

class CustomGenerator(xml.sax.saxutils.XMLGenerator):
    def startEndElement(self, name, attrs):
        self._write('<' + name)
        for (name, value) in attrs.items():
            self._write(' %s=%s' % (name, xml.sax.saxutils.quoteattr(value)))
        self._write(' />')
    def endDocument(self):
        self._out.close()

if __name__ == "__main__":
    import sys, os
    # IMPORTANT: Level file (first param or stdin) should be list of level name<linebreak>n level<linebreak>
    if len(sys.argv) < 2:
        f = sys.stdin
    else:
        f = open(sys.argv[1], 'U')
    os.mkdir("n.jmp")
    gen = CustomGenerator(open('n.jmp/index.xml', 'w'))
    gen.startDocument()
    gen.startElement('Jumpman', {'Version': '1.0'})
    isLevel = True
    i = 0
    for line in f:
        if isLevel:
            level = line[:-1]
            isLevel = False
        else:
            saveLevel(level, line[:-1])
            if i == 0:
                gen.startElement("Level", {"name": line[:-1], "flag": "1"})
            else:
                gen.startElement("Level", {"name": line[:-1]})
            gen.startEndElement("Type", {"camera": "0",
                                         "rots": "255",
                                         "dontrot": "0",
                                         "rspeed": "1",
                                         "zoom": "0.500000"})
            gen.startEndElement("Color", {"r": "1.000000", "g": "1.000000", "b": "1.000000"})
            safename = ''
            for c in line[:-1]:
                if c.isalnum() or (c == ' '): safename += c
            gen.startEndElement("File", {"src": safename+".png", "rot": "0"})
            #gen.startEndElement("File", {"src": safename+"2.png", "rot": "1"})
            gen.endElement("Level")
            isLevel = True
            i += 1
            if i > 3: i = 0
    gen.endElement('Jumpman')
    gen.endDocument()
